@@ -35,7 +35,7 @@
  * From: @(#)sys_term.c	5.16 (Berkeley) 3/22/91
  */
 char st_rcsid[] = 
-  "$Id: sys_term.c,v 1.12 1999/08/02 03:14:03 dholland Exp $";
+  "$Id: sys_term.c,v 1.17 1999/12/17 14:28:47 dholland Exp $";
 
 #include <utmp.h>
 
@@ -51,6 +51,9 @@ char st_rcsid[] =
 int openpty(int *, int *, char *, struct termios *, struct winsize *);
 #endif
 
+#define ARCH64 ((sizeof(void *)) == 8)
+#define ARCH32 ((sizeof(void *)) == 4)
+
 #if defined(AUTHENTICATE)
 #include <libtelnet/auth.h>
 #endif
@@ -215,34 +218,6 @@ int getpty(void) {
 	return -1;
     }
     return masterfd;
-
-#if 0
-    int p;
-    char *p1, *p2;
-    int i,j;
-
-    sprintf(line, "/dev/ptyXX");
-    p1 = &line[8];
-    p2 = &line[9];
-
-    for (i = 0; i < 16; i++) {
-	struct stat stb;
-
-	*p1 = "pqrstuvwxyzabcde"[i];
-	*p2 = '0';
-	if (stat(line, &stb) < 0)
-	    continue;
-	for (j = 0; j < 16; j++) {
-	    *p2 = "0123456789abcdef"[j];
-	    p = open(line, 2);
-	    if (p > 0) {
-		line[5] = 't';
-		return(p);
-	    }
-	}
-    }
-    return(-1);
-#endif
 }
 
 #ifdef	LINEMODE
@@ -627,12 +602,22 @@ void init_env(void) {
  * function will turn us into the login process.
  */
 
-static const char **addarg(const char **, const char *);
+struct argv_stuff {
+   const char **argv;
+   int argc;
+   int argmax;
+};
+
+static void addarg(struct argv_stuff *, const char *);
+static void initarg(struct argv_stuff *);
 
 void start_login(const char *host, int autologin, const char *name) {
-    const char **argv;
+    struct argv_stuff avs;
+    char *const *argvfoo;
     (void)autologin;
 
+    initarg(&avs);
+
     /*
      * -h : pass on name of host.
      *		WARNING:  -h is accepted by login if and only if
@@ -641,11 +626,11 @@ void start_login(const char *host, int autologin, const char *name) {
      *
      * -f : force this login, he has already been authenticated
      */
-    argv = addarg(0, loginprg);
-    argv = addarg(argv, "-h");
-    argv = addarg(argv, host);
+    addarg(&avs, loginprg);
+    addarg(&avs, "-h");
+    addarg(&avs, host);
 #if !defined(NO_LOGIN_P)
-    argv = addarg(argv, "-p");
+    addarg(&avs, "-p");
 #endif
 #ifdef BFTPDAEMON
     /*
@@ -653,8 +638,8 @@ void start_login(const char *host, int autologin, const char *name) {
      * to start bftp instead of shell.
      */
     if (bftpd) {
-	argv = addarg(argv, "-e");
-	argv = addarg(argv, BFTPPATH);
+	addarg(&avs, "-e");
+	addarg(&avs, BFTPPATH);
     } 
     else
 #endif
@@ -664,7 +649,7 @@ void start_login(const char *host, int autologin, const char *name) {
 	 * don't worry about the -f that might get sent.
 	 * A -s is supposed to override it anyhow.
 	 */
-	if (require_SecurID) argv = addarg(argv, "-s");
+	if (require_SecurID) addarg(&avs, "-s");
 #endif
 	if (*name=='-') {
 	    syslog(LOG_ERR, "Attempt to login with an option!");
@@ -673,15 +658,15 @@ void start_login(const char *host, int autologin, const char *name) {
 #if defined (AUTHENTICATE)
 	if (auth_level >= 0 && autologin == AUTH_VALID) {
 # if !defined(NO_LOGIN_F)
-	    argv = addarg(argv, "-f");
+	    addarg(&avs, "-f");
 # endif
-	    argv = addarg(argv, name);
+	    addarg(&avs, name);
 	} 
 	else
 #endif
 	{
 	    if (getenv("USER")) {
-		argv = addarg(argv, getenv("USER"));
+		addarg(&avs, getenv("USER"));
 		if (*getenv("USER") == '-') {
 		    write(1,"I don't hear you!\r\n",19);
 		    syslog(LOG_ERR,"Attempt to login with an option!");
@@ -691,36 +676,41 @@ void start_login(const char *host, int autologin, const char *name) {
 	}
     }
     closelog();
-    execv(loginprg, (char **) argv);
+    /* execv() should really take char const* const *, but it can't */ 
+    /*argvfoo = argv*/;
+    memcpy(&argvfoo, &avs.argv, sizeof(argvfoo));
+    execv(loginprg, argvfoo);
 
     syslog(LOG_ERR, "%s: %m\n", loginprg);
     fatalperror(net, loginprg);
 }
 
-static const char **addarg(const char **argv, const char *val) {
-    const char **cpp;
-    
-    if (argv == NULL) {
-	/*
-	 * 10 entries, a leading length, and a null
-	 */
-	argv = malloc(sizeof(*argv) * 12);
-	if (argv == NULL) return NULL;
-	*argv++ = (char *)10;
-	*argv = NULL;
-    }
-    for (cpp = argv; *cpp; cpp++);
-    if (cpp == &argv[(int)argv[-1]]) {
-	--argv;
-	*argv = (char *)((int)(argv[0]) + 10);
-	argv = realloc(argv, (int)(argv[0]) + 2);
-	if (argv == NULL) return NULL;
-	argv++;
-	cpp = &argv[(int)argv[-1] - 10];
-    }
-    *cpp++ = val;
-    *cpp = 0;
-    return(argv);
+static void initarg(struct argv_stuff *avs) {
+   /*
+    * 10 entries and a null
+    */
+   avs->argmax = 11;
+   avs->argv = malloc(sizeof(avs->argv[0]) * avs->argmax);
+   if (avs->argv == NULL) {
+      fprintf(stderr, "Out of memory\n");
+      exit(1);
+   }
+   avs->argc = 0;
+   avs->argv[0] = NULL;
+}
+
+static void addarg(struct argv_stuff *avs, const char *val) {
+   if (avs->argc>=avs->argmax-1) {
+       avs->argmax += 10;
+       avs->argv = realloc(avs->argv, sizeof(avs->argv[0])*avs->argmax);
+       if (avs->argv == NULL) {
+	  fprintf(stderr, "Out of memory\n");
+	  exit(1);
+       }
+   }
+
+   avs->argv[avs->argc++] = val;
+   avs->argv[avs->argc] = NULL;
 }
 
 /*
@@ -750,5 +740,5 @@ void cleanup(int sig) {
     chmod(line, 0666);
     chown(line, 0, 0);
     shutdown(net, 2);
-    exit(1);
+    exit(0);
 }
